// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/chart/spec/MockAnyChartStock",
	"cloudera/chart/ContextChartSet2"
], function(MockAnyChartStock, ContextChartSet2) {

describe("ContextChartSet2 Tests", function() {
	var chartSet;

	beforeEach(function() {
		var params = {
			startDate: new Date(1311909119001),
			endDate : new Date(1311910919001),
			mode: "INTERACTIVE"
		};

		chartSet = new ContextChartSet2(params, "testId");
	});

	describe("when the chartSet is created", function() {
		var config;
		beforeEach(function() {
			jasmine.Ajax.useMock();
			config = chartSet.getConfig();
		});

		it("should see 10 generalDataProviders.", function() {
			// We expect the time control to have configuration to have the following parameters.
			expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
			expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
			expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");
		});

		it("should see 1 scrollerDataProviders.", function() {
			var config = chartSet.getConfig();

			expect(config.data.dataProviders.scrollerDataProviders.length).toEqual(1);
		});

		it("should see 11 dataSets.", function() {
			var config = chartSet.getConfig();

			expect(config.data.dataSets.length).toEqual(11);
			expect(config.data.dataSets[0].id).toEqual("dataSet_0");
			expect(config.data.dataSets[10].id).toEqual("chartStretcher");
		});

		it("should see 0 chart.", function() {
			var config = chartSet.getConfig();
			expect(config.settings.charts.length).toEqual(0);
		});

		it("should format the units correctly.", function() {
			var allUnits = ["agents", "batches", "bytes", "events", "CPUs /s", "fds",
				"GB", "groups", "items", "jobs", "maps", "megabyte", "megabytes",
				"ms", "nodes", "pages", "percent", "records", "Records", "reduces",
				"seconds", "servers", "slots"];
			$.each(allUnits, function(index, units) {
				var value = chartSet.formatUnits(units);
				expect(value).toContain("{%Value}");
			});
		});

		it("should preprocess the csvdata with padding on both side correctly.", function() {
			chartSet.allowPadding = true;

			var csvData = "10,3\n15,5\n20,6\n";
			var startDate = new Date(5);
			var endDate = new Date(30);
			var padLeft = true;
			var padRight = true;
			var newData = chartSet.preprocessData(csvData, startDate, endDate, padLeft, padRight);
			expect(newData).toEqual("5,0\n9,0\n10,3\n15,5\n20,6\n21,0\n30,0\n");
		});

		it("should preprocess the csvdata with padding on the left correctly.", function() {
			chartSet.allowPadding = true;

			var csvData = "10,3\n15,5\n20,6\n";
			var startDate = new Date(5);
			var endDate = new Date(30);
			var padLeft = true;
			var padRight = false;
			var newData = chartSet.preprocessData(csvData, startDate, endDate, padLeft, padRight);
			expect(newData).toEqual("5,0\n9,0\n10,3\n15,5\n20,6\n");
		});

		it("should preprocess the csvdata with padding on the right correctly.", function() {
			chartSet.allowPadding = true;

			var csvData = "10,3\n15,5\n20,6\n";
			var startDate = new Date(5);
			var endDate = new Date(30);
			var padLeft = false;
			var padRight = true;
			var newData = chartSet.preprocessData(csvData, startDate, endDate, padLeft, padRight);
			expect(newData).toEqual("10,3\n15,5\n20,6\n21,0\n30,0\n");
		});

		it("should preprocess the csvdata with allowPadding = false correctly.", function() {
			chartSet.allowPadding = false;

			var csvData = "10,3\n15,5\n20,6\n";
			var startDate = new Date(5);
			var endDate = new Date(30);
			var padLeft = true;
			var padRight = true;
			var newData = chartSet.preprocessData(csvData, startDate, endDate, padLeft, padRight);
			expect(newData).toEqual(csvData);
		});

		it("should only apply event marker changes to charts with objectModels", function() {
			var oldChart = chartSet.chart;
			chartSet.chart = {
				applyEventMarkersChanges: jasmine.createSpy()
			};
			chartSet.applyEventMarkersChanges([]);
			expect(chartSet.chart.applyEventMarkersChanges).wasNotCalled();
			chartSet.chart = oldChart;
		});

	});

});
});
